/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2021 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 * 
 * SPDX-License-Identifier: EPL-2.0
 */
package de.cau.cs.kieler.kgraph.text.formatting;

import com.google.inject.Inject;
import de.cau.cs.kieler.kgraph.text.services.GRandomGrammarAccess;
import java.util.List;
import org.eclipse.xtext.Keyword;
import org.eclipse.xtext.formatting.impl.AbstractDeclarativeFormatter;
import org.eclipse.xtext.formatting.impl.FormattingConfig;
import org.eclipse.xtext.util.Pair;
import org.eclipse.xtext.xbase.lib.Extension;

@SuppressWarnings("all")
public class GRandomFormatter extends AbstractDeclarativeFormatter {
  @Inject
  @Extension
  private GRandomGrammarAccess _gRandomGrammarAccess;
  
  @Override
  protected void configureFormatting(final FormattingConfig c) {
    c.setAutoLinewrap(120);
    c.setLinewrap(0, 1, 2).before(this._gRandomGrammarAccess.getSL_COMMENTRule());
    c.setLinewrap(0, 1, 2).before(this._gRandomGrammarAccess.getML_COMMENTRule());
    c.setLinewrap(0, 1, 1).after(this._gRandomGrammarAccess.getML_COMMENTRule());
    List<Pair<Keyword, Keyword>> _findKeywordPairs = this._gRandomGrammarAccess.findKeywordPairs("{", "}");
    for (final Pair<Keyword, Keyword> pair : _findKeywordPairs) {
      {
        c.setIndentation(pair.getFirst(), pair.getSecond());
        c.setNoLinewrap().before(pair.getFirst());
        c.setLinewrap().after(pair.getFirst());
        c.setLinewrap().after(pair.getSecond());
      }
    }
    List<Keyword> _findKeywords = this._gRandomGrammarAccess.findKeywords("=");
    for (final Keyword equals : _findKeywords) {
      c.setNoLinewrap().before(equals);
    }
    c.setLinewrap().after(this._gRandomGrammarAccess.getDoubleQuantityRule());
    c.setLinewrap().after(this._gRandomGrammarAccess.getLabelsRule());
    c.setLinewrap().after(this._gRandomGrammarAccess.getIntegerRule());
    c.setLinewrap().after(this._gRandomGrammarAccess.getSTRINGRule());
    c.setLinewrap().after(this._gRandomGrammarAccess.getFormatsRule());
    c.setLinewrap().after(this._gRandomGrammarAccess.getFloatRule());
    c.setLinewrap().after(this._gRandomGrammarAccess.getSideRule());
    c.setLinewrap().after(this._gRandomGrammarAccess.getConstraintTypeRule());
    c.setLinewrap(2).after(this._gRandomGrammarAccess.getConfigurationRule());
  }
}
