/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://rtsys.informatik.uni-kiel.de/kieler
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 * 
 * SPDX-License-Identifier: EPL-2.0
 */
package de.cau.cs.kieler.klighd.ide.syntheses;

import com.google.common.collect.Iterables;
import de.cau.cs.kieler.klighd.ide.model.ErrorModel;
import de.cau.cs.kieler.klighd.ide.model.MessageModel;
import de.cau.cs.kieler.klighd.ide.syntheses.action.AbstractErrorModelShowExceptionAction;
import de.cau.cs.kieler.klighd.kgraph.KGraphData;
import de.cau.cs.kieler.klighd.kgraph.KNode;
import de.cau.cs.kieler.klighd.krendering.Colors;
import de.cau.cs.kieler.klighd.krendering.KRectangle;
import de.cau.cs.kieler.klighd.krendering.KRendering;
import de.cau.cs.kieler.klighd.krendering.KRoundedRectangle;
import de.cau.cs.kieler.klighd.krendering.KText;
import de.cau.cs.kieler.klighd.krendering.extensions.KContainerRenderingExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KRenderingExtensions;
import de.cau.cs.kieler.klighd.syntheses.AbstractDiagramSynthesis;
import javax.inject.Inject;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Diagram synthesis for a {@link MessageModel}.
 * 
 * @author als
 * @kieler.design 2014-07-30 proposed
 * @kieler.rating 2014-07-30 proposed yellow
 */
@SuppressWarnings("all")
public class ErrorModelSynthesis extends AbstractDiagramSynthesis<ErrorModel> {
  @Inject
  @Extension
  private KRenderingExtensions _kRenderingExtensions;
  
  @Inject
  @Extension
  private KContainerRenderingExtensions _kContainerRenderingExtensions;
  
  @Inject
  @Extension
  private MessageModelSynthesis _messageModelSynthesis;
  
  public static final String ID = "de.cau.cs.kieler.klighd.ui.view.syntheses.ErrorModelSynthesis";
  
  @Override
  public KNode transform(final ErrorModel model) {
    final KNode rootNode = this._messageModelSynthesis.transform(((MessageModel) model));
    if (((rootNode != null) && (!rootNode.getChildren().isEmpty()))) {
      final KNode messageNode = IterableExtensions.<KNode>head(rootNode.getChildren());
      KGraphData _head = IterableExtensions.<KGraphData>head(messageNode.getData());
      final KRectangle rect = ((KRectangle) _head);
      final KRoundedRectangle messageRect = IterableExtensions.<KRoundedRectangle>head(Iterables.<KRoundedRectangle>filter(rect.getChildren(), KRoundedRectangle.class));
      if ((messageRect != null)) {
        KRendering _head_1 = IterableExtensions.<KRendering>head(messageRect.getChildren());
        this._kRenderingExtensions.setForeground(_head_1, Colors.RED);
        String _stackTrace = model.getStackTrace();
        boolean _tripleNotEquals = (_stackTrace != null);
        if (_tripleNotEquals) {
          KText _addText = this._kContainerRenderingExtensions.addText(messageRect, "[Show Exception]");
          final Procedure1<KText> _function = new Procedure1<KText>() {
            @Override
            public void apply(final KText it) {
              ErrorModelSynthesis.this._kRenderingExtensions.setForeground(it, Colors.BLUE);
              ErrorModelSynthesis.this._kRenderingExtensions.setFontSize(it, 9);
              ErrorModelSynthesis.this._kRenderingExtensions.<KText>addSingleClickAction(it, AbstractErrorModelShowExceptionAction.ID);
              ErrorModelSynthesis.this._kRenderingExtensions.<KText>addDoubleClickAction(it, AbstractErrorModelShowExceptionAction.ID);
              ErrorModelSynthesis.this._kRenderingExtensions.to(ErrorModelSynthesis.this._kRenderingExtensions.from(ErrorModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it), ErrorModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, ErrorModelSynthesis.this._kRenderingExtensions.TOP, 4, 0), ErrorModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, ErrorModelSynthesis.this._kRenderingExtensions.BOTTOM, 8, 0);
            }
          };
          ObjectExtensions.<KText>operator_doubleArrow(_addText, _function);
        }
      }
    }
    return rootNode;
  }
}
