/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://rtsys.informatik.uni-kiel.de/kieler
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 * 
 * SPDX-License-Identifier: EPL-2.0
 */
package de.cau.cs.kieler.klighd.ide.syntheses;

import de.cau.cs.kieler.klighd.ide.model.MessageModel;
import de.cau.cs.kieler.klighd.kgraph.KNode;
import de.cau.cs.kieler.klighd.krendering.KImage;
import de.cau.cs.kieler.klighd.krendering.KRectangle;
import de.cau.cs.kieler.klighd.krendering.KRoundedRectangle;
import de.cau.cs.kieler.klighd.krendering.KText;
import de.cau.cs.kieler.klighd.krendering.extensions.KContainerRenderingExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KNodeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KRenderingExtensions;
import de.cau.cs.kieler.klighd.syntheses.AbstractDiagramSynthesis;
import de.cau.cs.kieler.klighd.syntheses.DiagramSyntheses;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Diagram synthesis for a {@link MessageModel}.
 * 
 * @author als
 * @kieler.design 2014-07-30 proposed
 * @kieler.rating 2014-07-30 proposed yellow
 */
@SuppressWarnings("all")
public class MessageModelSynthesis extends AbstractDiagramSynthesis<MessageModel> {
  @Inject
  @Extension
  private KNodeExtensions _kNodeExtensions;
  
  @Inject
  @Extension
  private KRenderingExtensions _kRenderingExtensions;
  
  @Inject
  @Extension
  private KContainerRenderingExtensions _kContainerRenderingExtensions;
  
  public static final String ID = "de.cau.cs.kieler.klighd.ui.view.syntheses.MessageModelSynthesis";
  
  @Override
  public KNode transform(final MessageModel model) {
    final KNode rootNode = this._kNodeExtensions.createNode();
    EList<KNode> _children = rootNode.getChildren();
    KNode _createNode = this._kNodeExtensions.createNode(model);
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode it) {
        KRectangle _addRectangle = MessageModelSynthesis.this._kRenderingExtensions.addRectangle(it);
        final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
          @Override
          public void apply(final KRectangle it) {
            MessageModelSynthesis.this._kRenderingExtensions.setInvisible(it, true);
            MessageModelSynthesis.this._kContainerRenderingExtensions.setGridPlacement(it, 1);
            boolean _hasIcon = model.hasIcon();
            if (_hasIcon) {
              KRectangle _addRectangle = MessageModelSynthesis.this._kContainerRenderingExtensions.addRectangle(it);
              final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
                @Override
                public void apply(final KRectangle it) {
                  MessageModelSynthesis.this._kRenderingExtensions.setInvisible(it, true);
                  MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it, model.getIconSize(), model.getIconSize());
                  MessageModelSynthesis.this._kContainerRenderingExtensions.addRectangularClip(MessageModelSynthesis.this._kRenderingExtensions.<KImage>setPointPlacementData(MessageModelSynthesis.this._kContainerRenderingExtensions.addImage(it, model.getIconPlugin(), model.getIconPath()), MessageModelSynthesis.this._kRenderingExtensions.LEFT, 0, 0.5f, MessageModelSynthesis.this._kRenderingExtensions.TOP, 0, 0.5f, 
                    MessageModelSynthesis.this._kRenderingExtensions.H_CENTRAL, MessageModelSynthesis.this._kRenderingExtensions.V_CENTRAL, 0, 0, model.getIconSize(), model.getIconSize()));
                }
              };
              ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
            }
            KRoundedRectangle _addRoundedRectangle = MessageModelSynthesis.this._kContainerRenderingExtensions.addRoundedRectangle(it, 7, 7);
            final Procedure1<KRoundedRectangle> _function_1 = new Procedure1<KRoundedRectangle>() {
              @Override
              public void apply(final KRoundedRectangle it) {
                MessageModelSynthesis.this._kContainerRenderingExtensions.setGridPlacement(it, 1);
                MessageModelSynthesis.this._kRenderingExtensions.setLineWidth(it, 2);
                String _title = model.getTitle();
                boolean _tripleNotEquals = (_title != null);
                if (_tripleNotEquals) {
                  KText _addText = MessageModelSynthesis.this._kContainerRenderingExtensions.addText(it, model.getTitle());
                  final Procedure1<KText> _function = new Procedure1<KText>() {
                    @Override
                    public void apply(final KText it) {
                      MessageModelSynthesis.this._kRenderingExtensions.setFontSize(it, 12);
                      MessageModelSynthesis.this._kRenderingExtensions.setFontBold(it, true);
                      MessageModelSynthesis.this._kRenderingExtensions.to(MessageModelSynthesis.this._kRenderingExtensions.from(MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it), MessageModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.TOP, 8, 0), MessageModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.BOTTOM, 4, 0);
                      DiagramSyntheses.suppressSelectability(it);
                    }
                  };
                  ObjectExtensions.<KText>operator_doubleArrow(_addText, _function);
                }
                String _message = model.getMessage();
                boolean _tripleNotEquals_1 = (_message != null);
                if (_tripleNotEquals_1) {
                  KText _addText_1 = MessageModelSynthesis.this._kContainerRenderingExtensions.addText(it, model.getMessage());
                  final Procedure1<KText> _function_1 = new Procedure1<KText>() {
                    @Override
                    public void apply(final KText it) {
                      MessageModelSynthesis.this._kRenderingExtensions.setFontSize(it, 12);
                      String _title = model.getTitle();
                      boolean _tripleNotEquals = (_title != null);
                      if (_tripleNotEquals) {
                        MessageModelSynthesis.this._kRenderingExtensions.to(MessageModelSynthesis.this._kRenderingExtensions.from(MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it), MessageModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.TOP, 0, 0), MessageModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.BOTTOM, 4, 0);
                      } else {
                        MessageModelSynthesis.this._kRenderingExtensions.to(MessageModelSynthesis.this._kRenderingExtensions.from(MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it), MessageModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.TOP, 8, 0), MessageModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.BOTTOM, 8, 0);
                      }
                      DiagramSyntheses.suppressSelectability(it);
                    }
                  };
                  ObjectExtensions.<KText>operator_doubleArrow(_addText_1, _function_1);
                }
              }
            };
            ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(_addRoundedRectangle, _function_1);
          }
        };
        ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
      }
    };
    KNode _doubleArrow = ObjectExtensions.<KNode>operator_doubleArrow(_createNode, _function);
    _children.add(_doubleArrow);
    return rootNode;
  }
}
